import React, {
  useState,
  useCallback,
  useEffect,
  useMemo,
} from 'react';
import {
  scaleTime,
  extent,
  scaleLog,
  max,
  min,
  line,
  timeFormat,
} from 'd3';
import { XAxis } from './XAxis';
import { YAxis } from './YAxis';
import { VoronoiOerlay } from './VoronoiOerlay';
import { Tooltip } from './Tooltip';
const xValue = (d) => d.date;
const yValue = (d) => d.deathTotal;

const margin = {
  left: 100,
  right: 80,
  top: 40,
  bottom: 60,
};
export const LineChart = ({ data, width, height }) => {
  const [activeRaw, setActiveRaw] = useState();

  const innerWidth = width - (margin.left + margin.right);
  const innerHeight = height - (margin.top + margin.bottom);
  const allData = useMemo(
    () =>
      data.reduce(
        (accumulator, countryTimeseries) =>
          accumulator.concat(countryTimeseries),
        []
      ),
    [data]
  );

  const epsilon = 1;

  const xScale = useMemo(
    () =>
      scaleTime()
        .domain(extent(allData, xValue))
        .range([0, innerWidth]),
    [allData, xValue]
  );

  const yScale = useMemo(
    () =>
      scaleLog()
        .domain([epsilon, max(allData, yValue)])
        .range([innerHeight, 0]),
    [epsilon, allData, yValue]
  );

  const lineGenerator = useMemo(
    () =>
      line()
        .x((d) => xScale(xValue(d)))
        .y((d) => yScale(epsilon + yValue(d))),
    [xScale, xValue, yScale, yValue, epsilon]
  );

  const handleVoronoiHover = useCallback(setActiveRaw, []);

  return (
    <svg width={width} height={height}>
      <g
        transform={`translate(${margin.left},${margin.top})`}
      >
        <VoronoiOerlay
          innerWidth={innerWidth}
          innerHeight={innerHeight}
          onHover={handleVoronoiHover}
          allData={allData}
          lineGenerator={lineGenerator}
        />
        <XAxis xScale={xScale} innerHeight={innerHeight} />
        <YAxis yScale={yScale} innerWidth={innerWidth} />
        {data.map((countryTimeseries, index) => {
          return (
            <path
              className="marker-line"
              d={lineGenerator(countryTimeseries)}
            />
          );
        })}
        {activeRaw
          ? [
              <path
                className="active"
                d={lineGenerator(
                  data.find(
                    (countryTimeseries) =>
                      countryTimeseries.countryName ===
                      activeRaw.countryName
                  )
                )}
              />,
              <g
                transform={`translate(
                ${lineGenerator.x()(activeRaw)}
                ,${lineGenerator.y()(activeRaw)})`}
              >
                <Tooltip
                  className="tooltip-title"
                  activeRaw={activeRaw}
                />
                <Tooltip
                  className="tooltip"
                  activeRaw={activeRaw}
                />
                <circle r={5} />
              </g>,
            ]
          : null}
        <text transform={`translate(${innerWidth / 3},0)`}>
          Global CoronaVirus Deaths Over Time by Country
        </text>
        <text
          text-anchor="middle"
          transform={`translate(-70,${
            innerHeight / 2
          }) rotate(-90)`}
        >
          Comulative Deaths
        </text>
        <text
          alignment-baseline="hanging"
          transform={`translate(${innerWidth / 2},${
            innerHeight + 30
          })`}
        >
          Time
        </text>
      </g>
    </svg>
  );
};
// <text>{activeRaw.countryName}</text>
